import React from 'react'

import {
  useDisclosure,
  AlertDialog,
  AlertDialogOverlay,
  AlertDialogContent,
  AlertDialogHeader,
  AlertDialogBody,
  AlertDialogFooter,
  Button,
  Text,
  Tooltip,
  ButtonProps,
  useToast,
} from '@chakra-ui/react'
import { useQueryClient } from 'react-query'

import { Pattern } from 'src/models'

import { DeletePatternMutation, GetPatternsQuery } from '../query-builder'
import { useLocation } from 'react-use'
import { navigate, routes } from '@redwoodjs/router'

export type RemovePatternButtonProps = {
  pattern: Pattern
  hasTooltip?: boolean
  children: React.ReactElement<ButtonProps>
}

export const RemovePatternButton: React.FC<RemovePatternButtonProps> = ({
  pattern,
  hasTooltip = false,
  children,
}) => {
  const toast = useToast()
  const queryClient = useQueryClient()
  const [isLoading, setIsLoading] = React.useState(false)
  const { isOpen, onOpen, onClose } = useDisclosure()
  const cancelRef = React.useRef<HTMLButtonElement>(null)
  const { pathname } = useLocation()

  const deletePatternMutation = DeletePatternMutation.useMutation({
    onSuccess: () => {
      setIsLoading(false)
      toast({
        status: 'success',
        position: 'top',
        title: `Se ha eliminado el patron ${pattern.name} satisfactoriamente.`,
      })
      queryClient.invalidateQueries(GetPatternsQuery.getKey({}))
      if (pathname.startsWith('/patterns/')) {
        // @ts-ignore -- TODO
        navigate(routes.patterns())
      }
    },
    onError: () => {
      toast({
        status: 'error',
        position: 'top',
        title: `Ha ocurrido un error al eliminar el patron.`,
      })
      onClose()
      setIsLoading(false)
    },
  })

  const handleDeletePattern = () => {
    setIsLoading(true)
    deletePatternMutation.mutate({ id: pattern.id })
  }

  if (!React.isValidElement(children) || Array.isArray(children)) {
    throw new Error(
      'RemovePatternButton child must be a single clickable element'
    )
  }

  const btn = React.cloneElement(children, { onClick: onOpen })

  return (
    <>
      <Tooltip isDisabled={!hasTooltip} label={<Text>Eliminar patron</Text>}>
        {btn}
      </Tooltip>
      <AlertDialog
        isOpen={isOpen}
        onClose={onClose}
        leastDestructiveRef={cancelRef}
      >
        <AlertDialogOverlay>
          <AlertDialogContent>
            <AlertDialogHeader fontSize="lg" fontWeight="bold">
              Eliminar este patron?
            </AlertDialogHeader>
            <AlertDialogBody>
              <Text>
                Si eliminas el patron <Text as="b">{pattern.name}</Text> todos
                sus datos asociados seran eliminados.
              </Text>
            </AlertDialogBody>
            <AlertDialogFooter>
              <Button ref={cancelRef} variant="ghost" onClick={onClose}>
                Cancelar
              </Button>
              <Button
                variant="solid"
                onClick={handleDeletePattern}
                ml="3"
                isLoading={isLoading}
              >
                Eliminar
              </Button>
            </AlertDialogFooter>
          </AlertDialogContent>
        </AlertDialogOverlay>
      </AlertDialog>
    </>
  )
}
